<?php
/**
 * Routes all the hooks to their respective actions and filters.
 *
 * @package StockSyncWithGoogleSheetForWooCommerce
 * @since 1.0.0
 */
// Namespace.
namespace StockSyncWithGoogleSheetForWooCommerce;

// Exit if accessed directly.
defined('ABSPATH') || exit();

if ( ! class_exists('\StockSyncWithGoogleSheetForWooCommerce\Hooks') ) {

	/**
	 * Routes all the hooks to their respective actions and filters.
	 *
	 * @package StockSyncWithGoogleSheetForWooCommerce
	 * @since 1.0.0
	 */
	final class Hooks extends Base {
		/**
		 * The single instance of the class.
		 *
		 * @var Hooks
		 */
		public static $instance = null;

		/**
		 * Initializes the class.
		 */
		public static function init() {
			if ( ! self::$instance ) {
				self::$instance = new self();
			}

			self::$instance->add_filters();
			self::$instance->add_actions();
		}


		/**
		 * Actions to be added for the plugin.
		 *
		 * @since 1.0.0
		 */
		public function add_actions() {

			$this->init_appsero_client();

			add_action('admin_menu', [ $this, 'add_admin_menu' ]);
			add_action('admin_init', [ $this, 'redirect_to_admin_page' ], 1);
			add_action('init', [ $this, 'check_ssgsw_synced' ], 99999);

			// Footer CSS for admin menu icon.
			add_action('admin_head', [ $this, 'admin_menu_icon_css' ]);

			// Admin enqueue scripts.
			add_action('admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ]);

			// Parse ID from Sheet URL and save for later use.
			add_action('ssgsw_updated_spreadsheet_url', [ $this, 'updated_spreadsheet_url_callback' ]);
			add_action('ssgsw_updated_show_sku', [ $this, 'ssgsw_sync_sheet_callback' ]);
			add_action('ssgsw_updated_show_short_description', [ $this, 'ssgsw_sync_sheet_callback' ]);
			add_action('ssgsw_updated_show_product_category', [ $this, 'ssgsw_sync_sheet_callback' ]);
			add_action('ssgsw_updated_show_attributes', [ $this, 'ssgsw_sync_sheet_callback' ]);
			add_action('ssgsw_updated_show_total_sales', [ $this, 'ssgsw_sync_sheet_callback' ]);

			/**
			 * Updating sheet hooks
			 */

			/**
			 * When a post type product is created or updated
			 */

			/**
			 * When stock is updated
			 */
			add_action('woocommerce_product_set_stock', [ $this, 'ssgsw_sync_sheet_callback' ], 10, 1);

			/**
			 * When product is moved to trash
			 */
			add_action('trashed_post', [ $this, 'trashed_post_callback' ], 10, 1);

			/**
			 * When product is restored from trash
			 */
			add_action('untrashed_post', [ $this, 'trashed_post_callback' ], 10, 1);

			/**
			 * When product is created, or updated or deleted
			 */
			add_action('save_post', [ $this, 'save_post_callback' ], 10, 3);
		}

		/**
		 * Filters to be added for the plugin.
		 *
		 * @since 1.0.0
		 */
		public function add_filters() {
			// Add promotional link to plugin action links.
			add_filter('plugin_action_links_' . plugin_basename(SSGSW_FILE), [ $this, 'add_plugin_action_links' ]);

			// Add promotional link to plugin meta links.
			add_filter('plugin_row_meta', [ $this, 'add_plugin_meta_links' ], 10, 2);

			add_filter('ssgsw_get_credentials', [ $this, 'ssgsw_get_credentials_callback' ]);

			add_filter('ssgs_get_column', [ $this, 'ssgsw_get_column_callback' ], 10, 3);
		}

		/**
		 * Add admin menu callback.
		 *
		 * @since 1.0.0
		 */
		public function add_admin_menu() {
			add_menu_page(
			__('Stock Sync with Google Sheet for WooCommerce', 'stock-sync-with-google-sheet-for-woocommerce'),
			__('Stock Sync with Google Sheet', 'stock-sync-with-google-sheet-for-woocommerce'),
			'manage_options',
			'ssgsw-admin',
			[ $this, $this->app->is_setup_complete() ? 'render_admin_page' : 'render_setup_page' ],
			SSGSW_PUBLIC . 'images/logo.svg',
			56
			);

			if ( ! $this->app->is_setup_complete() ) {
				add_submenu_page(
				'ssgsw-admin',
				__('Stock Sync with Google Sheet for WooCommerce', 'stock-sync-with-google-sheet-for-woocommerce'),
				__('Setup', 'stock-sync-with-google-sheet-for-woocommerce'),
				'manage_options',
				'ssgsw-admin',
				[ $this, 'render_setup_page' ],
				0
				);
			} else {
				add_submenu_page(
				'ssgsw-admin',
				__('Stock Sync with Google Sheet for WooCommerce', 'stock-sync-with-google-sheet-for-woocommerce'),
				__('Settings', 'stock-sync-with-google-sheet-for-woocommerce'),
				'manage_options',
				'ssgsw-admin',
				[ $this, 'render_admin_page' ],
				99
				);
			}
		}

		/**
		 * Render admin page callback.
		 *
		 * @since 1.0.0
		 */
		public function render_admin_page() {
			if ( $this->app->is_ultimate_activated() && ! $this->app->is_license_valid() ) {
				$this->load_template('dashboard/activate-license');
			}

			$this->load_template('dashboard/base');
		}

		/**
		 * Render setup page
		 */
		public function render_setup_page() {
			$this->app->reset_options(false);
			$this->load_template('setup/base');
		}


		/**
		 * Redirect to admin page
		 */
		public function redirect_to_admin_page() {
			$redirect_to_admin_page = ssgsw_get_option('redirect_to_admin_page', 0);

			if ( wp_validate_boolean( $redirect_to_admin_page ) ) {
				ssgsw_update_option('redirect_to_admin_page', 0);
				wp_safe_redirect( admin_url( 'admin.php?page=ssgsw-admin' ) );
				exit;
			}
		}

		/**
		 * Add plugin action links callback.
		 *
		 * @param array $links Plugin action links.
		 * @return array
		 */
		public function add_plugin_action_links( $links ) {

			if ( $this->app->is_setup_complete() ) {
				$links[] = '<a href="' . admin_url('admin.php?page=ssgsw-admin') . '">' . __('Settings', 'stock-sync-with-google-sheet-for-woocommerce') . '</a>';
			} else {
				$links[] = '<a href="' . admin_url('admin.php?page=ssgsw-admin') . '">' . __('Setup', 'stock-sync-with-google-sheet-for-woocommerce') . '</a>';
			}

			if ( ! $this->app->is_license_valid() ) {
				$links[] = wp_sprintf( '<a class="ssgsw-promo ssgsw-ultimate-button small" href="javascript:;"> <span class="ssgsw-ultimate-button">%s</span></a>', __('Get Ultimate', 'stock-sync-with-google-sheet-for-woocommerce') );
			}

			return $links;
		}

		/**
		 * Add plugin meta links callback.
		 *
		 * @param array  $links Plugin meta links.
		 * @param string $file Plugin file.
		 * @return array
		 */
		public function add_plugin_meta_links( $links, $file ) { //phpcs:ignore
			if ( ! plugin_basename( SSGSW_FILE ) ) {
				$links[] = wp_sprintf('<a target="_blank" href="https://wppool.dev/docs-category/stock-sync-with-google-sheet-for-woocommerce/"> <span class="dashicons dashicons-media-document" aria-hidden="true" style="font-size:16px;line-height:1.2"></span>%s</a>', __('Docs', 'stock-sync-with-google-sheet-for-woocommerce'));
				$links[] = wp_sprintf('<a target="_blank" href="https://wppool.dev/contact/"> <span class="dashicons dashicons-editor-help" aria-hidden="true" style="font-size:16px;line-height:1.2"></span>%s</a>', __('Support', 'stock-sync-with-google-sheet-for-woocommerce'));
			}
			return $links;
		}

		/**
		 * Admin menu icon css callback.
		 *
		 * @since 1.0.0
		 */
		public function admin_menu_icon_css() {
			printf('<style>%s</style>', '#adminmenu .toplevel_page_ssgsw-admin div.wp-menu-image img { 
				width: 18px;
				height: 18px;
			}');
		}


		/**
		 * Admin enqueue scripts callback.
		 *
		 * @param string $hook Current page hook.
		 * @since 1.0.0
		 */
		public function admin_enqueue_scripts( $hook ) {
			$pages = [ 'toplevel_page_ssgsw-admin', 'stock-sync-with-google-sheet_page_ssgsw-settings', 'edit.php', 'plugins.php' ];

			if ( ! in_array($hook, $pages) ) {
				return;
			}

			// check if we are on product edit page.
			if ( 'edit.php' === $hook && 'product' !== get_current_screen()->post_type ) {
				return;
			}

			wp_enqueue_style('ssgsw-admin-css', SSGSW_PUBLIC . 'css/admin.min.css', [], SSGSW_VERSION);
			wp_enqueue_script('ssgsw-admin-js', SSGSW_PUBLIC . 'js/admin.min.js', [ 'jquery' ], SSGSW_VERSION, true);

			// Localize script.
			wp_localize_script('ssgsw-admin-js', 'ssgsw_script', $this->app->localized_script());
		}

		/**
		 * Updated spreadsheet url callback.
		 *
		 * @param string $spreadsheet_url Spreadsheet url.
		 * @since 1.0.0
		 */
		public function updated_spreadsheet_url_callback( $spreadsheet_url ) {
			/**
			 * Get Sheet ID from Sheet URL Regex
			 */
			$sheet_id = preg_replace(' / ^ . * \ / d\ / ( . * )\ / . * $ / ', '$1', $spreadsheet_url);

			/**
			 * Get Sheet ID from Sheet URL
			 */

			if ( empty($sheet_id) ) {
				$sheet_id = preg_replace(' / ^ . * \ / d\ / ( . * )$ / ', '$1', $spreadsheet_url);

				if ( empty($sheet_id) ) {
					$sheet_id = preg_replace(' / ^ . * \ / ( . * )$ / ', '$1', $spreadsheet_url);
				}
			}

			error_log('Spreadsheet ID: ' . $sheet_id);
			ssgsw_update_option('spreadsheet_id', $sheet_id);
		}

		/**
		 * Get credentials callback.
		 *
		 * @param string $credentials Credentials.
		 * @return array
		 */
		public function ssgsw_get_credentials_callback( $credentials ) {
			$credentials = json_decode($credentials, true);

			return array_map('wp_unslash', $credentials);
		}


		/**
		 * Get column callback.
		 *
		 * @param mixed  $value Column value.
		 * @param string $key Column key.
		 * @param object $row Row object.
		 * @return mixed
		 */
		public function ssgsw_get_column_callback( $value, $key, $row ) {

			$column = new Column();

			if ( '_stock' === $key ) {

				if ( isset($row->_stock) && is_numeric($row->_stock) && $row->_stock > 0 ) {
					return absint( $row->_stock );
				}

				if ( isset($row->_stock_status) ) {
					return $column->get_stock_status($row->_stock_status);
				}
			}

			// ID.
			if ( 'ID' === $key || 'total_sales' === $key ) {
				return absint( $value );
			}

			// Price.
			if ( in_array( $key, [ '_sale_price', '_regular_price', '_price' ] ) ) {
				if ( $value > 0 ) {
					return round($value, 2);
				}
				return '';
			}

			// Product Category.
			if ( 'product_cat' === $key ) {
				return $column->get_items_by_comma($value);
			}

			// Product type.
			if ( 'product_type' === $key ) {
				return $column->get_product_type($value);
			}

			if ( 'post_excerpt' === $key ) {
				return $row->_variation_description ?? $value;
			}

			if ( '_product_attributes' === $key ) {

				if ( $value ) {
					$value = maybe_unserialize($value);

					$attributes = [];

					foreach ( $value as $attribute ) {
						if ( $attribute['is_taxonomy'] ) {
							$taxonomy_values = wp_get_post_terms( $row->ID, $attribute['name'], [ 'fields' => 'slugs' ] );
							$attributes[]    = $attribute['name'] . ': ' . implode(' | ', $taxonomy_values);
						} elseif ( ! empty($attribute['value']) ) {
							$attributes[] = $attribute['name'] . ': ' . $attribute['value'];
						}
					}

					$attributes = array_filter($attributes);

					return implode(', ', $attributes);
				}

				$attributes = array_filter( (array) $row, function ( $key ) {
					return strpos($key, 'attribute_') !== false;
				}, ARRAY_FILTER_USE_KEY);

				$attributes = array_map(function ( $key, $value ) {
					if ( $value ) {
						return str_replace('attribute_', '', $key) . ': ' . $value;
					} else {
						return '';
					}
				}, array_keys($attributes), $attributes);

				$attributes = array_filter($attributes);

				return implode(', ', $attributes);
			}

			return (string) $value;
		}


		/**
		 * Save option freeze headers callback.
		 *
		 * @param mixed $value Freeze headers value.
		 * @return void
		 */
		public function ssgsw_save_option_freeze_headers_callback( $value ) {
			$sheet   = new Sheet();
			$updated = $sheet->freeze_headers( true === wp_validate_boolean( $value ) );
			$this->send_json($updated);
		}

		/**
		 * Sync sheet callback.
		 *
		 * @return void
		 */
		public function ssgsw_sync_sheet_callback() {

			if ( isset( $GLOBALS['ssgs_sync_all_products'] ) && true === $GLOBALS['ssgs_sync_all_products'] ) {
				return;
			}

			$product = new Product();
			$product->sync_all();
		}

		/**
		 * Initiates Appsero Client.
		 *
		 * @return mixed
		 */
		public function init_appsero_client() {

			if ( ! class_exists( '\Appsero\Client' ) ) {
				require_once SSGSW_PATH . ' / appsero / src / Client . php';
			}

			// appsero_is_local NOT.
			add_filter( 'appsero_is_local', '__return_false' );

			$client = new \Appsero\Client(
			'2153b02c - 08d6 - 45e0 - 8295 - 6afc39509fe5',
			'Stock Sync with Google Sheet for WooCommerce',
			SSGSW_FILE
			);

			// Active insights.
			$client->insights()->init();

			// Active automatic updater.
			$client->updater();

			// Init WPPOOL Plugin.
			if ( function_exists( 'wppool_plugin_init' ) ) {
				wppool_plugin_init( 'stock_sync_with_google_sheet_for_woocommerce' );
			}
		}

		/**
		 * Trashed post callback.
		 *
		 * @param int $post_id Post ID.
		 * @return void
		 */
		public function trashed_post_callback( $post_id ) {
			if ( 'product' === get_post_type($post_id) ) {
				$this->ssgsw_sync_sheet_callback();
			}
		}




		/**
		 * Checks if ssgsw synced.
		 *
		 * @return void
		 */
		public function check_ssgsw_synced() {
			$ssgsw_synced = wp_validate_boolean( get_option('ssgsw_synced') );

			if ( $ssgsw_synced ) {
				return;
			}
			update_option('ssgsw_synced', true);
			$product = new Product();
			$product->sync_all();

		}


		/**
		 * Saves post callback.
		 *
		 * @param int      $post_id Post ID.
		 * @param \WP_Post $post Post object.
		 * @param bool     $update Whether this is an existing post being updated or not.
		 * @return void
		 */
		public function save_post_callback( $post_id, $post, $update ) { // phpcs:ignore
			if ( 'product' === get_post_type($post_id) || 'product_variation' === get_post_type($post_id) || 'shop_order' === get_post_type($post_id) ) {
				$this->ssgsw_sync_sheet_callback();
			}
		}


	}

	// Initiate the class.
	Hooks::init();
}
