<?php
/**
 * Product Model for Stock Sync with Google Sheet for WooCommerce.
 *
 * @package StockSyncWithGoogleSheetForWooCommerce
 * @since 1.0.0
 */
// Namespace.
namespace StockSyncWithGoogleSheetForWooCommerce;

// Exit if accessed directly.
defined('ABSPATH') || exit;

if ( ! class_exists('\StockSyncWithGoogleSheetForWooCommerce\Product') ) {

	/**
	 * Product Model for Stock Sync with Google Sheet for WooCommerce.
	 *
	 * @package StockSyncWithGoogleSheetForWooCommerce
	 * @since 1.0.0
	 */
	final class Product extends Base {

		/**
		 * Utilities Trait to use in all classes globally.
		 */
		use Utilities;

		/**
		 * Get raw data from database.
		 *
		 * @param array $ids Product ids.
		 * @return mixed
		 */
		public function get_raw_data( array $ids = null ) {
			global $wpdb;

			$column = new Column();

			$fields = $column->get_queryable_fields();

			$main_table = $wpdb->prefix . 'posts';

			$query  = 'SELECT ';
			$query .= implode(', ', array_map(function ( $field ) use ( $main_table ) {
				return "$main_table.$field";
			}, $fields));

			/**
			 * From post meta table
			 */
			$post_meta_table = $wpdb->prefix . 'postmeta';
			$meta_fields     = $column->get_queryable_metas();

			if ( $meta_fields && count($meta_fields) > 0 ) {
				$query .= ', ';

				$meta_query = [];

				foreach ( $meta_fields as $meta_field_key => $meta_field ) {
					$meta_field_key_name = str_replace('-', '_', $meta_field_key);
					$meta_query[]        = "(SELECT {$post_meta_table}.meta_value FROM $post_meta_table WHERE $post_meta_table.post_id = $main_table.ID AND $post_meta_table.meta_key " . ( $meta_field['compare'] ?? '=' ) . " '$meta_field_key' LIMIT 1) as $meta_field_key_name";
				}

				$query .= implode(', ', $meta_query);
			}

			/**
			 * From taxonomy table
			 */

			$taxonomy_fields = $column->get_queryable_taxonomies();

			if ( $taxonomy_fields && count($taxonomy_fields) > 0 ) {
				$query .= ', ';
				$query .= implode(', ', array_map(function ( $field ) use ( $main_table, $wpdb ) {
					return "(SELECT GROUP_CONCAT(CONCAT_WS(':' , t.term_id, t.name)) FROM $wpdb->term_relationships tr INNER JOIN $wpdb->term_taxonomy tt ON tr.term_taxonomy_id = tt.term_taxonomy_id INNER JOIN $wpdb->terms t ON t.term_id = tt.term_id WHERE tr.object_id = $main_table.ID AND tt.taxonomy = '$field' LIMIT 1)  as $field";
				}, $taxonomy_fields));

					error_log( print_r( $taxonomy_fields, true ) );
			}

				$relations = $column->get_queryable_relations();

				// order lookup.
				$status = apply_filters( 'ssgswc_wc_order_product_lookup_status', [ 'wc-completed', 'wc-processing', 'wc-hold', 'wc-refunded' ] );

			foreach ( $relations as $relation ) {

				switch ( $relation ) {
					case 'total_sales':
						$status = implode("','", $status);
						$query .= ", (SELECT SUM( order_item_meta__qty.meta_value ) FROM {$wpdb->prefix}woocommerce_order_itemmeta AS order_item_meta__qty
                    INNER JOIN {$wpdb->prefix}woocommerce_order_items AS order_items ON order_items.order_item_id = order_item_meta__qty.order_item_id
                    INNER JOIN {$wpdb->prefix}posts AS posts ON posts.ID = order_items.order_id
                    WHERE posts.post_status IN ( '$status' ) AND posts.post_type = 'shop_order' AND order_item_meta__qty.meta_key = '_qty' AND order_item_meta__qty.order_item_id = order_items.order_item_id AND order_items.order_item_type = 'line_item' AND order_items.order_item_id IN ( SELECT order_item_id FROM {$wpdb->prefix}woocommerce_order_itemmeta AS order_item_meta__product_id WHERE order_item_meta__product_id.meta_key IN ( '_product_id', '_variation_id' ) AND order_item_meta__product_id.meta_value = $main_table.ID ) ) AS total_sales";

						break;

					default:
						break;
				}
			}

				$query .= " From $main_table";

				// Includes variations.
				$query .= " WHERE $main_table.post_type IN ('product', 'product_variation' )";

				// Not deleted.
				$query .= " AND $main_table.post_status = 'publish' AND $main_table.post_status != 'trash'";

				// Conditional by in ID.
			if ( $ids && count($ids) > 0 ) {
				$query .= " AND $main_table.ID IN (" . implode(',', $ids) . ')';
			}

				// Avoid duplicating IDs.
				$query .= " GROUP BY $main_table.ID ";

				// Order and Order By.
				$order    = esc_sql( apply_filters( 'ssgs_product_order', 'ASC' ) );
				$order_by = esc_sql( apply_filters( 'ssgs_product_order_by', 'post_date' ) );

				$query .= "ORDER BY $main_table.$order_by $order";

				/**
				 * Set limit
				 */
				$limit = apply_filters('ssgsw_product_limit', 20);
			if ( $limit ) {
				$query .= $wpdb->prepare( ' LIMIT %d', $limit );
			}

			// phpcs:ignore
			$results = $wpdb->get_results( $query ); // db call ok; no-cache ok.

			if ( $wpdb->last_error ) {
				return $wpdb->last_error;
			}

				return $results;
		}

		/**
		 * Get Formatted Data.
		 *
		 * @param array $ids Product IDs.
		 * @return mixed
		 */
		public function get_formatted_data( $ids = null ) {
			$raw_data = $this->get_raw_data($ids);

			if ( ! $raw_data || count($raw_data) === 0 ) {
				return false;
			}

			$column = new Column();
			$keys   = $column->get_column_keys();
			error_log( print_r( $keys, true ) );

			$formatted_data = array_map(function ( $row ) use ( $keys ) {
				$formatted_row = [];
				foreach ( $keys as $key ) {
					$formatted_row[ $key ] = apply_filters('ssgs_get_column', $row->$key, $key, $row);
				}
				if ( '_stock_status' === $key ) {
					unset($formatted_row[ $key ]);
				}
				return $formatted_row;
			}, $raw_data);

			$formatted_data = array_map(function ( $row ) {
				return array_values( (array) $row);
			}, $formatted_data);

			return $formatted_data;
		}

		/**
		 * Get All Products.
		 *
		 * @return array
		 */
		public function get_all_products() {
			$formatted_data = $this->get_formatted_data();

			$columns = new Column();
			$headers = $columns->get_column_names();

			if ( $formatted_data && count($formatted_data) > 0 ) {
				array_unshift($formatted_data, $headers);
			}

			return $formatted_data;
		}

		/**
		 * Sync all products.
		 *
		 * @return mixed
		 */
		public function sync_all() {

			if ( ! $this->app->is_plugin_ready() ) {
				return __('Plugin is not ready to use.', 'stock-sync-with-google-sheet-for-woocommerce');
			}

			$data = $this->get_all_products();

			if ( ! $data || count($data) === 0 ) {
				return __('No products found.', 'stock-sync-with-google-sheet-for-woocommerce');
			}

			$google_sheet = new Sheet();

			$updated = $google_sheet->update_values('A1', $data);

			return wp_validate_boolean( $updated );
		}

		/**
		 * Update bulk products from sheet.
		 *
		 * @param array $products Products.
		 * @return bool
		 **/
		public function bulk_update( array $products = [] ) {

			$GLOBALS['ssgs_sync_all_products'] = true;
			$column                            = new Column();

			// Checks if plugin is ready to use.
			$add_products_from_sheet = wp_validate_boolean(apply_filters('ssgsw_add_products_from_sheet', ssgsw_get_option('add_products_from_sheet')));

			foreach ( $products as $data ) {

				$data = (object) $data;

				$product = null;

				if ( isset($data->ID) && $data->ID > 0 ) {
					$product = wc_get_product($data->ID);
				} else {

					if ( ! $add_products_from_sheet ) {
						return false;
					}

					// create new product type post.
					$product = new \WC_Product();
				}

				if ( ! $product ) {
					return false;
				}

				if ( ! $product ) {
					continue;
				}

				$is_variation = $product->is_type('variation');
				$is_variable  = $product->is_type('variable');

				$variable  = $is_variable ? $product : null;
				$variation = $is_variation ? $product : null;

				if ( $is_variation ) {
					$variation = new \WC_Product_Variation($product->get_id());
				}

				if ( $is_variable ) {
					$variable = new \WC_Product_Variable($product->get_id());
				}

				/** Update name */
				if ( isset($data->name) ) {
					$product->set_name($data->name);
				}

				/** Update excerpt */
				if ( isset($data->post_excerpt) ) {
					if ( true === $is_variation ) {
						$product->set_description($data->post_excerpt);
					} else {
						$product->set_short_description($data->post_excerpt);
					}
				}

				/** Update regular price */
				if ( isset( $data->regular_price ) ) {
					if ( $data->regular_price > 0 ) {
						$product->set_regular_price($data->regular_price);
					} else {
						$product->set_regular_price('');
					}
				}

				/** Update sale price */
				if ( isset($data->sale_price) ) {
					if ( $data->sale_price > 0 ) {
						$product->set_sale_price($data->sale_price);
					} else {
						$product->set_sale_price('');
					}
				}

				// Support for regularPrice and salePrice till next major version.
				if ( isset( $data->regularPrice ) ) { // phpcs:ignore
					if ( $data->regularPrice > 0 ) { // phpcs:ignore
						$product->set_regular_price($data->regularPrice); // phpcs:ignore
					} else {
						$product->set_regular_price('');
					}
				}

				if ( isset($data->salePrice) ) { // phpcs:ignore
					if ( $data->salePrice > 0 ) { // phpcs:ignore
						$product->set_sale_price($data->salePrice); // phpcs:ignore
					} else {
						$product->set_sale_price('');
					}
				}

				/** Update SKU */
				if ( isset($data->sku) && $data->sku !== $product->get_sku() ) {
					$product->set_sku($data->sku);
				}

				if ( isset($data->stock) ) {

					if ( is_numeric($data->stock) ) {
						$product->set_manage_stock(true);
						if ( $data->stock > 0 ) {
							$product->set_stock_quantity( (int) $data->stock);
						} else {
							$product->set_stock_quantity(0);
						}
					} else {
						$product->set_manage_stock(false);
						$stock_status = $column->get_stock_status($data->stock, true);
						$product->set_stock_status($stock_status);
					}
				}

				$product->save();
			}

			return $this->sync_all();
		}
	}
}
