<?php
/**
 * Sheet class for Stock Sync with Google Sheet for WooCommerce.
 *
 * @package StockSyncWithGoogleSheetForWooCommerce
 * @since 1.0.0
 */

// Namespace.
namespace StockSyncWithGoogleSheetForWooCommerce;

// Exit if accessed directly.
defined('ABSPATH') || exit;

// Use Google API.
use Google\Service\Analytics\Columns;


if ( ! class_exists('\StockSyncWithGoogleSheetForWooCommerce\Sheet') ) {

	/**
	 * Sheet class for Stock Sync with Google Sheet for WooCommerce.
	 *
	 * @package StockSyncWithGoogleSheetForWooCommerce
	 * @since 1.0.0
	 */
	class Sheet extends Base {
		/**
		 * Credentials for Google API.
		 *
		 * @var array
		 */
		protected $credentials;

		/**
		 * Spreadsheet URL
		 *
		 * @var string
		 */
		protected $spreadsheet_url;

		/**
		 * Spreadsheet ID
		 *
		 * @var string
		 */
		protected $spreadsheet_id;

		/**
		 * Spreadsheet Tab ID
		 *
		 * @var string
		 */
		protected $sheet_id;

		/**
		 * Spreadsheet Tab Name
		 *
		 * @var string
		 */
		protected $sheet_tab;

		/**
		 * Constructor.
		 *
		 * @param string $sheet_id Spreadsheet ID.
		 * @param string $sheet_tab Spreadsheet Tab.
		 * @throws \Exception If plugin is not ready to use.
		 */
		public function __construct( $sheet_id = null, $sheet_tab = null ) {

			/**
			 * Check if plugin is ready to use
			 */

			if ( ssgsw()->is_plugin_ready() === false ) {
				throw new \Exception('Plugin is not ready to use.');
			}

			/**
			 * Default credentials
			 */
			$this->credentials = ssgsw_get_option('credentials');

			/**
			 * The Spreadsheet
			 */
			$this->spreadsheet_url = ssgsw_get_option('spreadsheet_url');

			$this->spreadsheet_id = $sheet_id ?? ssgsw_get_option('spreadsheet_id');

			if ( ! $this->spreadsheet_id ) {
				// Parse Spreadsheet ID from URL using regex.
				preg_match('/spreadsheets\/d\/([a-zA-Z0-9-_]+)/', $this->spreadsheet_url, $matches);
				$this->spreadsheet_id = $matches[1] ?? null;
			}

			/**
			 * Single Sheet
			 */
			$this->sheet_tab = $sheet_tab ?? ssgsw_get_option('sheet_tab');
			$this->sheet_id  = $sheet_tab ?? ssgsw_get_option('sheet_id');
		}

		/**
		 * Set Sheet ID.
		 *
		 * @param string $spreadsheet_id Spreadsheet ID.
		 * @return $this
		 */
		public function setID( $spreadsheet_id = null ) {
			if ( $spreadsheet_id ) {
				$this->spreadsheet_id = $spreadsheet_id;
			}

			return $this;
		}

		/**
		 * Set Sheet Tab Name.
		 *
		 * @param string $sheet_tab Spreadsheet Tab.
		 * @return $this
		 */
		public function setTab( $sheet_tab = null ) {
			if ( $sheet_tab ) {
				$this->sheet_tab = $sheet_tab;
			}

			return $this;
		}

		/**
		 * Instantiate Google Cloud API Client Service.
		 *
		 * @return mixed
		 */
		protected function GoogleService() {

			try {
				$client = new \Google_Client();
				$client->setApplicationName('Stock Sync With Google Sheet For Woocommerce 2');
				$client->setScopes(\Google_Service_Sheets::SPREADSHEETS);
				$client->setAuthConfig($this->credentials);
				$client->setAccessType('offline');

				$service = new \Google_Service_Sheets($client);

				return $service;
			} catch ( \Exception $e ) {

				return $e->getMessage();
			}
		}

		/**
		 * Get values from google sheet by range.
		 *
		 * @param string $range Range.
		 * @param string $dimension Dimension.
		 * @param string $sheet_tab Sheet Tab.
		 * @return array|bool
		 */
		public function get_values( $range = null, $dimension = 'ROWS', $sheet_tab = null ) {

			if ( ! $range ) {
				return false;
			}

			if ( ! $sheet_tab ) {
				$sheet_tab = $this->sheet_tab;
			} else {
				$this->sheet_tab = $sheet_tab;
			}

			try {
				$range    = $this->sheet_tab . '!' . $range;
				$service  = $this->GoogleService();
				$response = $service->spreadsheets_values->get($this->spreadsheet_id, $range, [ 'majorDimension' => $dimension ]);
				$values   = $response->getValues();
				return $values;
			} catch ( \Throwable $error ) {
				return false;
			}
		}

		/**
		 * Get rows from google sheet by range.
		 *
		 * @param string $range Range.
		 * @param string $sheet_tab Sheet Tab.
		 */
		public function get_rows( $range = null, $sheet_tab = null ) {
			if ( ! $range ) {
				return false;
			}
			return $this->get_values($range, 'ROWS', $sheet_tab);
		}

		/**
		 * Get columns from google sheet by range.
		 *
		 * @param string $range Range.
		 * @param string $sheet_tab Sheet Tab.
		 * @return array|bool
		 */
		public function get_columns( $range = null, $sheet_tab = null ) {
			if ( ! $range ) {
				return false;
			}
			return $this->get_values($range, 'COLUMNS', $sheet_tab);
		}

		/**
		 * Updates values in google sheet by range.
		 *
		 * @param string $range Range.
		 * @param array  $values Values.
		 * @param string $dimension Dimension.
		 * @return mixed
		 */
		public function update_values( $range = null, $values = null, $dimension = null ) {

			if ( ! $range || ! $values ) {
				return false;
			}

			try {

				$service        = $this->GoogleService();
				$clear_range    = $this->sheet_tab . '!A1:Z';
				$clear_body     = new \Google_Service_Sheets_ClearValuesRequest();
				$clear_response = $service->spreadsheets_values->clear($this->spreadsheet_id, $clear_range, $clear_body);

				$body = new \Google_Service_Sheets_ValueRange([
					'values' => $values,
				]);

				$params = [
					'valueInputOption' => 'RAW',
				];

				if ( $dimension ) {
					$body->setMajorDimension($dimension);
				}

				$range    = $this->sheet_tab . '!' . $range;
				$response = $service->spreadsheets_values->update($this->spreadsheet_id, $range, $body, $params);

				return $response;
			} catch ( \Throwable $error ) {
				return $error;
			}
		}

		/**
		 * Updates rows in google sheet by range.
		 *
		 * @param string $range Range.
		 * @param array  $values Values.
		 * @return mixed
		 */
		public function update_row_values( $range = null, $values = null ) {

			if ( ! $range || ! $values ) {
				return false;
			}

			return $this->update_values($range, $values, 'ROWS');
		}

		/**
		 * Updates columns in google sheet by range.
		 *
		 * @param string $range Range.
		 * @param array  $values Values.
		 * @return mixed
		 */
		public function update_row_columns( $range = null, $values = null ) {
			if ( ! $range || ! $values ) {
				return false;
			}
			return $this->update_values($range, $values, 'COLUMNS');
		}



		/**
		 * Initializes the Google Sheets API service.
		 *
		 * @throws \Exception If the API client library is not found.
		 * @return mixed
		 */
		public function initialize() {

			$service = $this->GoogleService();

			try {

				$response = $service->spreadsheets->get($this->spreadsheet_id);

				try {
					$sheets = $response->getSheets();
				} catch ( \Throwable $error ) {
					return $error;
				}

				$sheet = array_filter($sheets, function ( $sheet ) {
					return $sheet->properties->title === $this->sheet_tab;
				});

				/**
				 * Getting Sheet ID of working sheet
				 */
				if ( ! $sheet ) {

					// if no sheet title matched, create new one with the title of the sheet.
					$response = $this->create_sheet_tab($this->sheet_tab);

					$sheet = $response->replies[0]->addSheet;
				} else {
					$sheet = array_values($sheet)[0];
				}

				/**
				 * Save working Sheet ID to database for later use.
				 */

				ssgsw_update_option('sheet_id', $sheet->properties->sheetId);

				$updated = $this->sync_sheet_headers();
				return $updated;

			} catch ( \Exception $e ) {
				throw new \Exception(__('Unable to access Google Sheet. Please check required permissions.', 'stock-sync-with-google-sheet-for-woocommerce'));
			}
		}

		/**
		 * Creates a new sheet tab.
		 *
		 * @param string $sheet_name Sheet Name.
		 */
		public function create_sheet_tab( $sheet_name = null ) {
			if ( ! $sheet_name ) {
				$sheet_name = $this->sheet_tab;
			}

			$service = $this->GoogleService();

			try {

				$response = $service->spreadsheets->batchUpdate($this->spreadsheet_id, new \Google_Service_Sheets_BatchUpdateSpreadsheetRequest([
					'requests' => [
						new \Google_Service_Sheets_Request([
							'addSheet' => new \Google_Service_Sheets_AddSheetRequest([
								'properties' => new \Google_Service_Sheets_SheetProperties([
									'title' => $sheet_name,
								]),
							]),
						]),
					],
				]));

				return $response;
			} catch ( \Exception $e ) {
				return false;
			}
		}

		/**
		 * Syncs sheet headers
		 *
		 * @return mixed
		 */
		public function sync_sheet_headers() {
			try {
				$column   = new Column();
				$keys     = $column->get_column_names();
				$response = $this->update_row_values('A1', [ $keys ]);

				return $response;
			} catch ( \Exception $e ) {
				return false;
			}
		}

		/**
		 * Resets sheet.
		 *
		 * @return mixed
		 */
		public function reset_sheet() {
			try {
				// clear all values.
				$service      = $this->GoogleService();
				$request_body = new \Google_Service_Sheets_ClearValuesRequest();
				$response     = $service->spreadsheets_values->clear($this->spreadsheet_id, $this->sheet_tab . '!A2:Z', $request_body);

				return $response;
			} catch ( \Exception $e ) {
				return $e;
			}
		}

		/**
		 * Freezes headers.
		 *
		 * @param boolean $freeze Freeze.
		 * @return mixed
		 */
		public function freeze_headers( $freeze = true ) {
			try {
				$service      = $this->GoogleService();
				$request_body = new \Google_Service_Sheets_UpdateSheetPropertiesRequest([
					'properties' => new \Google_Service_Sheets_SheetProperties([
						'sheetId'        => $this->sheet_id,
						'gridProperties' => new \Google_Service_Sheets_GridProperties([
							'frozenRowCount'    => $freeze ? 1 : 0,
							'frozenColumnCount' => $freeze ? 1 : 0,
						]),
					]),
					'fields'     => 'gridProperties.frozenRowCount,gridProperties.frozenColumnCount',
				]);

				$response = $service->spreadsheets->batchUpdate($this->spreadsheet_id, new \Google_Service_Sheets_BatchUpdateSpreadsheetRequest([
					'requests' => [
						new \Google_Service_Sheets_Request([
							'updateSheetProperties' => $request_body,
						]),
					],
				]));
				return $response;
			} catch ( \Exception $e ) {
				return $e;
			}
		}
	}
}
